#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_sharepoint_too
----------------------------------

Tests for `sharepoint_too` module.
"""

import unittest

from sharepoint_too import sharepoint_too


class TestSharepoint_too(unittest.TestCase):

    def setUp(self):
        pass

    def test_something(self):
        pass

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()
